@@ -1,7 +1,8 @@
 '''
 Contains CS basic operations and pipelines
 '''
-import os, sys
+import os
+import sys
 from tqdm import tqdm
 import numpy as np
 import pandas as pd
